#!/usr/local/bin/perl5
# Usage:
#
#	/scripts/jrun.pl/servlet/SnoopServlet
# or
#	/cgi-bin/jrun.pl/servlet/SnoopServlet
#
# depending on how your server is configured.
#

#
# JRun CGI Connector - Connect to JRun from any CGI environment
# 
# Install this perl script anywhere you can run CGI scripts.
# The procedures for doing this are specific to each web server.
# Assuming the script is installed in "cgi-bin", you can invoke
# servlets as follows:
#
#    http://host/cgi-bin/jrun.pl/servlet/SnoopServlet
#
# In this case, /servlet/SnoopServlet is passed to JRun, who
# applies mapping and alias rules, and invokes the servlet.  The
# output is returned by the CGI script.  Here is another example
#
#    http://host/cgi-bin/jrun.pl/yourpage.jsp
#
# This will invoke /yourpage.jsp on JRun and return the results.
#
# CONFIGURATION
#
# This script looks at the JRUNPROXY environment variable to determine
# how to connect to JRun.  The value must be in the format:
#
#    <IP address>:<port>
#
# the default value is: 127.0.0.1:8081
#
# CUSTOMIZATION
#
# The script is designed to be easily customized by the customer.  The
# JRun proxy address can be changed below, and the error responses can
# be edited by modifying the appropriate subroutine at the bottom of
# the script.  The subroutines are:
#
#     ConnectFailed     called when we can't connect to JRun
#     SocketFailed      called when we cannot get a new socket from the OS
#     SyswriteFailed    called upon protocol error
#     BadHost           called when we can't determine JRun's hostname
#
#


# Use socket library
use Socket;

$SIG{ALRM} = sub { die(1); };

# The JRun proxy settings from JRUNPROXY environment variable or localhost
if($ENV{'JRUNPROXY'} ne "") {
	$jrun_proxy = $ENV{'JRUNPROXY'};
}
else
{
	$jrun_proxy = "127.0.0.1:8081";
}

# Get the POST/PUT data
if($ENV{'REQUEST_METHOD'} eq "GET") {
	# nothing
} else {
	$clen = $ENV{'CONTENT_LENGTH'};
	if ($clen > 0) {
		read(STDIN, $postbuffer, $clen);
	}
}

eval {
   # contributed by sstidman@uucom.com
   #
   # The following line sets the alarm to kill the Perl Script
   # if the servlet does not complete in the alotted time.

   alarm(300);
   sendrequest();

   # The following line turns off the alarm and returns the amount
   # of time left on the previous alarm
   # If the value of $@ is 0, then the alarm went off
   alarm(0);
};

sub sendrequest
{
	local($jrun_socket,$jrun_port,$jrun_host,@jrun_addrs,$jrun_addr);
	local($headers,$name,$aliases,$protocol,$addrtype,$length);

	@jrun_parts = split( /:/, $jrun_proxy );

	$jrun_host = $jrun_parts[0];
	$jrun_port = $jrun_parts[1];
	
	# find the address for the server
	if ($jrun_host =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/)
	{
		$jrun_addr = pack("C4", $1, $2, $3, $4);
	}
	else
	{
		($name,$aliases,$addrtype,$length,@jrun_addrs) = 
			gethostbyname($jrun_host) || (&BadHost, return %result);
		$jrun_addr = $jrun_addrs[0];
	}
		
	# set up the socket to talk to the payment server

	# find the TCP protocol information
	($name,$aliases,$protocol) = getprotobyname('tcp');

	# Put the socket info into the correct format for the connect
    $jrun_socket = pack('S n a4 x8',AF_INET,$jrun_port,$jrun_addr);

	# make the socket filehandle and connect to the socket
	# if any fail, reply and return with an appropriate error message
	# "S" is the socket filehandle
    socket(S,AF_INET,SOCK_STREAM,$protocol) || (&SocketFailed, return %result);

	# set nodelay on the socket    (ignore errors)
    setsockopt( S, IPPROTO_TCP, TCP_NODELAY, 1);	
	
	# connect
    connect(S,$jrun_socket) || (&ConnectFailed,exit %result);

	$len1 = length( $ENV{'PATH_INFO'} );
	$len2 = length( $ENV{'SCRIPT_NAME'} );
	$uri = $ENV{'PATH_INFO'};
	if (substr($uri, 0, $len2) eq $ENV{'SCRIPT_NAME'}) {
		$uri = substr( $ENV{'PATH_INFO'}, $len2, $len1-$len2 );
	}
	$headers="URI: " . $uri . "\r\n";
	
	while ( ($key,$val) = each(%ENV) )
	{
		$headers = $headers . $key . ": " . $val . "\r\n";
	}

	# Send initial command
	sendcommand( 0 );
	
	# Send headers
	sendbytes( $headers, length( $headers ) );

	# Init $command
	$command = 0;
	
	# Loop for requests
	while( $command != 9 )
	{
		$command = getcommand();
		alarm(0);  # reset the alarm

		if ( $@ ) 
		{
		   # We reached this point because the alarm went off 
		   # so we need to send an error message
		   if ( $command != 8) 
		   {
			print(STDOUT "Content-type: text/html\n\n");
			print(STDOUT "<HTML><HEAD><TITLE>Timeout</TITLE></HEAD><BODY>Timeout occurred running servlet.</BODY></HTML>\n");
		   }
		}


		if( $command == 3 )
		{
			$name = getbytes();
			$value = $ENV{$name};
			$valuelen = length( $value );
			
			if( $valuelen == 0 )
			{
				$name = "HTTP_" . $name;
				$value = $ENV{$name};
				$valuelen = length( $value );
			}
			
			if( $valuelen == 0 )
			{
				sendbytes( "", 0 );
			}
			else
			{
				sendbytes( $value, $valuelen );
			}
		}

		if( $command == 4 )
		{
			$name = getbytes();
			$val = $ENV{'PATH_TRANSLATED'};
			$len = length( $val );
			sendbytes( $val, $len );
		}

		if( $command == 5 )
		{
			sendbytesnolen( $postbuffer, $clen );
		}
		
		if( $command == 6 )
		{
			# get status code
			$len = sysread( S, $codebytes, 4 );
			$code = unpack("N", $codebytes);
			$codestr = getbytes();
		}
		
		if( $command == 7 )
		{
			print STDOUT "Connection: close\r\n";   # this may be overkill
			$| = 1; # flush after each print from now on
			print STDOUT getbytes();  # headers from jrun
		}
		
		if( $command == 8 )
		{
			$data = getbytes();
			# Windows Perl converts \n to \r\n
			#$data =~ s/\r//g;
			syswrite(STDOUT, $data, length($data));
		}
		
	}
	
	close(S);	

	exit(0);
}

sub getcommand
{
	$len = sysread( S, $bytes, 4 );
	return unpack("N", $bytes);
}

sub getbytes
{	
	$len = sysread( S, $bytes, 4 );
	
	$len = unpack("N", $bytes);
	$offset = 0;
	
	$| = 1;
	while($len > 0)
	{
		$readin = sysread( S, $bytes, $len, $offset );
		unless ( defined($readin) && $readin > 0 ) { &SysreadFailed; exit %result; }
		$len -= $readin;
		$offset += $readin;
	}
	
	return $bytes;
}

sub sendcommand
{
	local($command) = @_;

	# force flush the output buffer
	select(S); $| = 1; print ''; select(STDOUT);

	# write command
	$write_result = syswrite( S, pack("N", $command ), 4);
	unless ( defined($write_result) ) { &SyswriteFailed; exit %result; }

	# reinstate buffered I/O
    $| = 0;
}
	

sub sendbytes
{
	local($bytes, $bytes_len) = @_;
	
	# force flush the output buffer
	select(S); $| = 1; print ''; select(STDOUT);

	# send length as 4 byte int	
	sendcommand( $bytes_len );

	if( $bytes_len > 0 )
	{
		$offset = 0;
		
		while($bytes_len)
		{
			# send the message over the socket
			$write_result = syswrite( S, $bytes, $bytes_len, $offset );

			# $write_result will be undef if there was an error
			unless ( defined($write_result) ) { &SyswriteFailed; exit %result; }
			
			$bytes_len -= $write_result;
			$offset += $write_result;
		}
	}
	
	# reinstate buffered I/O
    $| = 0;
}

sub sendbytesnolen
{
	local($bytes, $bytes_len) = @_;
	
	# force flush the output buffer
	select(S); $| = 1; print ''; select(STDOUT);

	if( $bytes_len > 0 )
	{
		$offset = 0;
		
		while($bytes_len > 0)
		{
			# send the message over the socket
			$write_result = syswrite( S, $bytes, $bytes_len, $offset );

			# $write_result will be undef if there was an error
			unless ( defined($write_result) ) { &SyswriteFailed; exit %result; }
			
			$bytes_len -= $write_result;
			$offset += $write_result;
		}
	}
	
	# reinstate buffered I/O
    $| = 0;
}

#
# ConnectFailed: error handler when we can't connect to JRun.
# You can replace this error with a redirect to a fancy error page.
# See the example below.
#
sub ConnectFailed
{
	# comment this out if you use use a redirect instead
	print STDOUT "Content-type: text/html\r\n\r\n";
	printf STDOUT "<h2>Could not connect to JRun Connector Proxy: <b>%s</b></h2>\r\n", $jrun_proxy;
	print STDOUT "<h3>Please contact the system administrator for this web site</h3>\r\n";

	# example redirect on error
	#print STDOUT "302 Could not connect to JRun Connector Proxy\r\n";
	#print STDOUT "Location: http://www.your-site-here.com/connect-error.html\r\n\r\n";
}

#
# SocketFailed: Called when we failed to obtain a socket.
# Indicates resource starvation.
#
sub SocketFailed
{
	#print STDOUT "503 Could not get socket for JRun Proxy connection\r\n";
	print STDOUT "Content-type: text/html\r\n\r\n";
	print STDOUT "<h2>Could not get socket for JRun Proxy connection</h2>\r\n";
	print STDOUT "<h3>Please contact the system administrator for this web site</h3>\r\n";

	# example redirect on error
	#print STDOUT "302 Could not get socket\r\n";
	#print STDOUT "Location: http://www.your-site-here.com/socket-error.html\r\n\r\n";
}

#
# SyswriteFailed: called when we have a write error on the socket
# Indicates a protocol error or JRun is busy (very rare)
#
sub SyswriteFailed
{
	#print STDOUT "503 JRun Proxy protocol error\r\n";
	print STDOUT "Content-type: text/html\r\n\r\n";
	print STDOUT "<h2>JRun Proxy protocol error<h2>\r\n";
	print STDOUT "<h3>Please contact the system administrator for this web site</h3>\r\n";

	# example redirect on error
	#print STDOUT "302 JRun proxy write error\n";
	#print STDOUT "Location: http://www.your-site-here.com/write-error.html\r\n\r\n";
}

sub BadHost
{
	#print STDOUT "503 Invalid hostname for JRun Connector Proxy\r\n";
	print STDOUT "Content-type: text/html\r\n\r\n";
	printf STDOUT "<h2>Invalid hostname for JRun Connector Proxy: <b>%s</b>\r\n", $jrun_proxy;
	print STDOUT "<h3>Please contact the system administrator for this web site</h3>\r\n";

	# example redirect on error
	#print STDOUT "302 Could not connect to JRun Connector Proxy\r\n";
	#print STDOUT "Location: http://www.your-site-here.com/write-error.html\r\n\r\n";
}


sub SysreadFailed
{
	#print STDOUT "503 JRun Proxy dropped connection\r\n";
	print STDOUT "Content-type: text/html\r\n\r\n";
	print STDOUT "<h2>JRun Proxy dropped connection<h2>\r\n";
	print STDOUT "<h3>Please contact the system administrator for this web site</h3>\r\n";

	# example redirect on error
	#print STDOUT "302 JRun dropped connection\n";
	#print STDOUT "Location: http://www.your-site-here.com/write-error.html\r\n\r\n";
}

