/*
Copyright (c) 1997-2000 Allaire.  All Rights Reserved.

DO NOT REDISTRIBUTE THIS SOFTWARE IN ANY WAY WITHOUT THE EXPRESSED
WRITTEN PERMISSION OF ALLAIRE.
*/

#include "extension.h"
#include "jrun_property.h"

// Globals
const char *		scriptPath = NULL;

BOOL WINAPI
GetFilterVersion(HTTP_FILTER_VERSION *pVer)
{
	jrun_request jr, *r = &jr;
	init_isapi_request(r, NULL, NULL);

	// Set the version (keep it at 2.0)
	pVer->dwFilterVersion = HTTP_FILTER_REVISION;

	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	// Set the flags we are interested in
	pVer->dwFlags |= SF_NOTIFY_ORDER_HIGH | SF_NOTIFY_SECURE_PORT |
		SF_NOTIFY_NONSECURE_PORT | SF_NOTIFY_PREPROC_HEADERS | SF_NOTIFY_LOG;

	// Load description string
	lstrcpyn(pVer->lpszFilterDesc, "JRun ISAPI Filter", SF_MAX_FILTER_DESC_LEN);

	r->log(r, "%s ISAPI Filter %s - %s %s", JRUN_APP_NAME, JRUN_VERSION,
		__DATE__, __TIME__);

	return TRUE;
}


DWORD WINAPI
HttpFilterProc(HTTP_FILTER_CONTEXT *pfc,
			    DWORD notificationType, 
			    LPVOID pvNotification)
{
	jrun_request jr, *r = &jr;
	init_isapi_request(r, NULL, pfc);	

	__try {

		HTTP_FILTER_PREPROC_HEADERS *pHeaderInfo = (HTTP_FILTER_PREPROC_HEADERS *) pvNotification;

		r->trace(r, "In filter: notification = 0x%08X", notificationType);

		switch (notificationType) {
		case SF_NOTIFY_PREPROC_HEADERS: {
			extern jrun_proxy jrunProxy;
			char buf[8192];

			// http says GET parameter must be <= 1024
			DWORD len = sizeof(buf);
			pHeaderInfo->GetHeader(pfc, "url", buf, &len);
			if (len >= 1024) {
				r->trace(r, "big url %d",len);
			}
			char *origRequest = (char *) pfc->AllocMem(pfc, len, 0);
			strcpy(origRequest, buf);

			r->trace(r, "JRun Filter got url: '%s'", origRequest);

			// 2-15-98 Added by Martin Franklin - MediaDNA Inc to support true logging
			// Copy the original requested url and put it into the filter context
			pfc->pFilterContext = (void *)origRequest;

			// this is not exactly right (breaks with '.' in dir name)
			// but its the same as older versions of the connector.
			char *truncRequest = (char *) pfc->AllocMem(pfc, len+1, 0);
			strcpy(truncRequest, origRequest);
			char *p = strchr(truncRequest, '?');
			if (p != NULL) {
				*p = '\0'; // truncate query string, if any
			}
			r->uri = truncRequest;

			// map the request
			jrun_proxy *target = jrunProxy.filterRequest(&jrunProxy, r);

			// if they are different (new and old) then it was remapped
			if(target != NULL) {
				LPSTR newRequest = (LPSTR) pfc->AllocMem(pfc,
						(DWORD) (strlen(scriptPath) + strlen(origRequest) + 2), 0);
				strcpy(newRequest, scriptPath);
				strcat(newRequest, origRequest);
				r->trace(r, "JRun Filter mapped url to: '%s'.", newRequest);
				pHeaderInfo->SetHeader(pfc, "url",	newRequest);
			}
			break;
		}

		// 2-15-98 Added by Martin Franklin - MediaDNA Inc to support true logging
		case SF_NOTIFY_LOG: {
			HTTP_FILTER_LOG *pvData = (HTTP_FILTER_LOG *)pvNotification;
			r->trace(r,"Log notify");
			// When logging occurs reset the target URL back the way it was
			char *origRequest = (char *) pfc->pFilterContext;
			if (origRequest != NULL) {
				pvData->pszTarget= origRequest;
			}
			break;
		}
		}

	} __except(EXCEPTION_EXECUTE_HANDLER) {

		int code = GetExceptionCode();

		if(code == EXCEPTION_ACCESS_VIOLATION)
			r->log(r, "An ACCESS VIOLATION occured in JRun Connector filter!" );
		else
			r->log(r, "An exception occured in JRun Connector filter! (%ld)", code);
	}

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}
